
#include <windows.h>  
#include <commctrl.h>
#include "makedl.h"  


#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif

#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32

HINSTANCE hInst;   // current instance

LPCTSTR lpszAppName = "MyApp";
LPCTSTR lpszTitle   = "MakeDragList()"; 


BOOL RegisterWin95( CONST WNDCLASS* lpwc );


int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG      msg;
   HWND     hWnd; 
   WNDCLASS wc;

   wc.style         = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon (hInstance, lpszAppName); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              

   if ( IS_WIN95 )
   {
      if ( !RegisterWin95( &wc ) )
         return( FALSE );
   }
   else if ( !RegisterClass( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}


BOOL RegisterWin95( CONST WNDCLASS* lpwc )
{
   WNDCLASSEX wcex;

   wcex.style         = lpwc->style;
   wcex.lpfnWndProc   = lpwc->lpfnWndProc;
   wcex.cbClsExtra    = lpwc->cbClsExtra;
   wcex.cbWndExtra    = lpwc->cbWndExtra;
   wcex.hInstance     = lpwc->hInstance;
   wcex.hIcon         = lpwc->hIcon;
   wcex.hCursor       = lpwc->hCursor;
   wcex.hbrBackground = lpwc->hbrBackground;
   wcex.lpszMenuName  = lpwc->lpszMenuName;
   wcex.lpszClassName = lpwc->lpszClassName;

   // Added elements for Windows 95.
   //...............................
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.hIconSm = LoadImage(wcex.hInstance, lpwc->lpszClassName, 
                            IMAGE_ICON, 16, 16,
                            LR_DEFAULTCOLOR );
			
   return RegisterClassEx( &wcex );
}


LPCTSTR lpszData[8] = { "Apple", "Orange", "Pear", "Watermeleon", 
                        "Grape", "Peach", "Strawberry", "Rasberry" };


LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
static LPDRAGLISTINFO lpdli;
static int  nItem;
static int  nItemToMove = -1;
static UINT uDLMsg      = 0;
static HWND hList       = NULL;

   // Handle the dragging notifications.
   //...................................
   if ( uMsg == uDLMsg )
   {
      lpdli = (LPDRAGLISTINFO)lParam;

      switch (lpdli->uNotification) 
      {
         case DL_BEGINDRAG:
                 nItemToMove = LBItemFromPt(lpdli->hWnd, lpdli->ptCursor, TRUE);
                 return( TRUE );

         case DL_DRAGGING:
                 nItem = LBItemFromPt(lpdli->hWnd, lpdli->ptCursor, TRUE);
                 DrawInsert( hWnd, lpdli->hWnd, nItem);

                 if ( nItem != -1 ) 
                 {
                    SetCursor( LoadCursor( hInst, "DragCursor" ) ); 
                    return( 0 );
                 }

                 return( DL_STOPCURSOR );

         case DL_CANCELDRAG :
                 nItemToMove = -1;
                 break;

         case DL_DROPPED:
                 nItem = LBItemFromPt(lpdli->hWnd, lpdli->ptCursor, TRUE);
                 if ( nItem != -1 )
                 {
                    char szBuf[40];

                    SendMessage( hList, LB_GETTEXT, nItemToMove, (LPARAM)szBuf );
                    SendMessage( hList, LB_DELETESTRING, nItemToMove, 0 );
                    SendMessage( hList, LB_INSERTSTRING, nItem, (LPARAM)szBuf );
                    SendMessage( hList, LB_SETCURSEL, nItem, 0 );
                 }

                 DrawInsert(hWnd, lpdli->hWnd, -1);
                 nItemToMove = -1;

                 return( DL_CURSORSET );
      }
   }

   switch( uMsg )
   {
      case WM_CREATE  :
              InitCommonControls();

              hList = CreateWindow( "LISTBOX", "",
                                     WS_CHILD | WS_VISIBLE | LBS_STANDARD | 
                                     LBS_NOINTEGRALHEIGHT,
                                     10, 100, 100, 100,
                                     hWnd,
                                     (HMENU)1,
                                     hInst,
                                     NULL);
              if ( hList )
              {
                 int i;

                 for( i=0; i<8; i++ )
                    SendMessage( hList, LB_INSERTSTRING, (WPARAM)-1, 
                                                         (LPARAM)lpszData[i] );

                 MakeDragList( hList );
                 uDLMsg = RegisterWindowMessage( DRAGLISTMSGSTRING );
              }
              break;

      case WM_SIZE :
              if ( hList )
                 MoveWindow( hList, 0, 0, LOWORD( lParam ), HIWORD( lParam ), FALSE );
              break;

      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, (DLGPROC)About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;
      
      case WM_DESTROY :
              PostQuitMessage(0);
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );
}


LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}
